"""
Carpeta Tributaria Electrónica (CTE)
====================================

Official FAQ:
http://www.sii.cl/preguntas_frecuentes/carp_trib_electronica/preg_carp_trib_electronica.htm


¿La Carpeta Tributaria Electrónica [..] puede ser utilizada como certificado?

> Sí, la Carpeta Tributaria Electrónica de Internet puede ser utilizada como
> certificado.
> El SII, por su parte, garantiza que la información contenida en la carpeta
> sea vigente y válida para la fecha y hora en que sea generada.

Source: http://www.sii.cl/preguntas_frecuentes/carp_trib_electronica/001_045_5787.htm


¿Qué es y para qué sirve la Carpeta Tributaria Electrónica [..]?

> La Carpeta Tributaria Electrónica permitirá entregar la información
> tributaria de un contribuyente a un tercero autorizado, con el objeto de
> solicitar créditos bancarios, acreditar rentas, entre otras.

Source: http://www.sii.cl/preguntas_frecuentes/carp_trib_electronica/001_045_5776.htm
"""
